/*==============================================================================*/
/*  Copyright (C) 2009-2016, Heartland Data inc. All Rights Reserved.           */
/*                                                                              */
/*  Title  :   SPI Fast Driver                                                  */
/*  EventID:   Insert EventID Test Point                                        */
/*  AppVer :   DT10 Ver10.XX`@                                                */
/*  FileID :   14C1                                                             */
/*  Version:   1.1                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Please customize the code for your environment.                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for Port Control                                             */
/*==============================================================================*/
#include "Common.h"
/* #include "driverlib/ssi.h" */

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT10 Project setting.    */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Macro:  DT_INLINE                                                           */
/*  Desc:   Please use "static" instead of "inline" if "inline" cannot be used. */
/*==============================================================================*/
#define	DT_INLINE	inline
/* #define	DT_INLINE	static */

/*==============================================================================*/
/*  Func:   _TP_BusPortInit                                                     */
/*  Desc:   Please describe the code to initializes ports.                      */
/*==============================================================================*/
static void _TP_BusPortInit(void)
{
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOA );
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOD );
	SysCtlPeripheralEnable(SYSCTL_PERIPH_SSI0);

	GPIOPinTypeGPIOOutput( GPIO_PORTD_BASE, GPIO_PIN_3 );		/* CS */
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_3, GPIO_PIN_3);

	GPIOPinConfigure(0x00000801);				/* GPIO_PA2_SSI0CLK */
	GPIOPinConfigure(0x00001001);				/* GPIO_PA4_SSI0RX  */
	GPIOPinConfigure(0x00001401);				/* GPIO_PA5_SSI0TX  */
	GPIOPinTypeSSI(GPIO_PORTA_BASE, GPIO_PIN_5 | GPIO_PIN_4 | GPIO_PIN_2);
	SSIConfigSetExpClk(SSI0_BASE, SysCtlClockGet(), SSI_FRF_MOTO_MODE_3, SSI_MODE_MASTER, 25000000, 8);
	SSIEnable(SSI0_BASE);
}

/*==============================================================================*/
/*  Func:   portSetCS                                                           */
/*  Desc:   Please describe the code to set the value to CS Port.               */
/*==============================================================================*/
DT_INLINE void portSetCS(DT_UINT dat)
{
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_3, dat?0xff:0x00 );
}

/*==============================================================================*/
/*  Func:   enterCritical                                                       */
/*  Desc:   Please describe the code to enter Critical Section.                 */
/*==============================================================================*/
DT_INLINE void enterCritical(void)
{
	taskENTER_CRITICAL();
}

/*==============================================================================*/
/*  Func:   exitCritical                                                        */
/*  Desc:   Please describe the code to exit Critical Section.                  */
/*==============================================================================*/
DT_INLINE void exitCritical(void)
{
	taskEXIT_CRITICAL();
}

/*==============================================================================*/
/*  Don't change the code from here as possible.                                */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define		DT_VARIABLE_BIT			0x02
#define		DT_EVTTRG_BIT			0x08
#define		DT_VARIABLE_FAST_BIT	0x01
#define		DT_EVTTRG_FAST_BIT		0x02


/*==============================================================================*/
/*  Func:   portInit                                                            */
/*  Desc:   Initialize Port Function                                            */
/*==============================================================================*/
static int init = 0;
DT_INLINE void portInit(void)
{
	if( init == 0 ){
		_TP_BusPortInit();
		init = 1;
	}
}

/*==============================================================================*/
/*  Func: _TP_BusOutDrv                                                         */
/*  Desc: Test Point Output Function                                            */
/*==============================================================================*/
DT_INLINE void _TP_BusOutDrv( DT_UINT addr, DT_UINT dat )
{
	unsigned long temp;

	portSetCS(0);
	/* data part 16bit */
	SSIDataPut(SSI0_BASE, dat >> 8);
	while( SSIBusy(SSI0_BASE) );	/* Transmission waiting */
	SSIDataGet(SSI0_BASE, &temp);	/* Dummy Read */
	SSIDataPut(SSI0_BASE, dat);
	while( SSIBusy(SSI0_BASE) );	/* Transmission waiting */
	SSIDataGet(SSI0_BASE, &temp);	/* Dummy Read */
	/* address part 16bit */
	SSIDataPut(SSI0_BASE, addr >> 8);
	while( SSIBusy(SSI0_BASE) );	/* Transmission waiting */
	SSIDataGet(SSI0_BASE, &temp);	/* Dummy Read */
	SSIDataPut(SSI0_BASE, addr);
	while( SSIBusy(SSI0_BASE) );	/* Transmission waiting */
	SSIDataGet(SSI0_BASE, &temp);	/* Dummy Read */
	portSetCS(1);
}

/*==============================================================================*/
/*  Func:   _TP_BusOutByteDrv                                                   */
/*  Desc:   Byte Data Output Function                                           */
/*==============================================================================*/
DT_INLINE void _TP_BusOutByteDrv( DT_UINT dat )
{
	unsigned long temp;
	SSIDataPut(SSI0_BASE, dat);
	while( SSIBusy(SSI0_BASE) );	/* Transmission waiting */
	SSIDataGet(SSI0_BASE, &temp);	/* Dummy Read */
}

/*==============================================================================*/
/*  Func:   _TP_BusOutFastDrv                                                   */
/*  Desc:   Test Point Output(Fast) Function                                    */
/*==============================================================================*/
DT_INLINE void _TP_BusOutFastDrv( DT_UINT step, DT_UINT bit_num )
{
	portSetCS(0);
	if( bit_num >= 9 ) _TP_BusOutByteDrv( step >> 8 );
	_TP_BusOutByteDrv( step );
	portSetCS(1);
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutDrv                                                    */
/*  Desc:   Value Output Function                                               */
/*==============================================================================*/
DT_INLINE void _TP_MemoryOutDrv( unsigned char *p, DT_UINT size )
{
	portSetCS(0);
	if( size >= 256 ) size = 256;
	_TP_BusOutByteDrv( size );
	for( ; size != 0; --size, ++p ){
		_TP_BusOutByteDrv( *p );
	}
	portSetCS(1);
}

/*==============================================================================*/
/*  Func: _TP_BusOut                                                            */
/*  Desc: Called by Test Point                                                  */
/*==============================================================================*/
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{
	portInit();
	enterCritical();
	
	_TP_BusOutDrv( addr, dat );
	
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable Test Point                                       */
/*==============================================================================*/
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size )
{
	portInit();
	enterCritical();
	
	_TP_BusOutDrv( addr | DT_VARIABLE_BIT, dat );
	
	_TP_MemoryOutDrv( (unsigned char *)value, size );
	
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_EventTrigger                                                    */
/*  Desc:   Called by Event Trigger 12bit Test Point                            */
/*==============================================================================*/
void _TP_EventTrigger( DT_UINT addr, DT_UINT dat, DT_UINT event_id )
{
	portInit();
	enterCritical();

	_TP_BusOutDrv( addr | DT_EVTTRG_BIT, dat );

	portSetCS(0);

	_TP_BusOutByteDrv(event_id);
	_TP_BusOutByteDrv((event_id >> 8) & 0x0f);

	portSetCS(1);

	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_EventTrigger32                                                  */
/*  Desc:   Called by Event Trigger 32bit Test Point                            */
/*==============================================================================*/
void _TP_EventTrigger32( DT_UINT addr, DT_UINT dat, DT_UINT event_id )
{
	portInit();
	enterCritical();

	_TP_BusOutDrv( addr | DT_EVTTRG_BIT, dat );

	portSetCS(0);

	_TP_BusOutByteDrv(event_id);
	_TP_BusOutByteDrv(event_id >> 8);
	_TP_BusOutByteDrv(event_id >> 16);
	_TP_BusOutByteDrv(event_id >> 24);

	portSetCS(1);
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_BusOutFast                                                      */
/*  Desc:   Called by Fast Test Point                                           */
/*==============================================================================*/
void _TP_BusOutFast( DT_UINT step, DT_UINT bit_num )
{
	portInit();
	step = step << 2;
	bit_num = bit_num + 2;
	enterCritical();
	_TP_BusOutFastDrv( step, bit_num );
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutputFast                                                */
/*  Desc:   Called by Variable Fast Test Point                                  */
/*==============================================================================*/
void _TP_MemoryOutputFast( DT_UINT step, DT_UINT bit_num, void *value, DT_UINT size )
{
	portInit();
	step = (step << 2) + DT_VARIABLE_FAST_BIT;
	bit_num = bit_num + 2;
	enterCritical();
	_TP_BusOutFastDrv( step, bit_num );
	portSetCS(0);
	_TP_MemoryOutDrv( (unsigned char *)value, size );
	portSetCS(1);
	exitCritical();
}

/*==============================================================================*/
/*  Func: _TP_EventTriggerFast                                                  */
/*  Desc: Called by Event Trigger Fast Test Point                               */
/*==============================================================================*/
void _TP_EventTriggerFast( DT_UINT step, DT_UINT bit_num, DT_UINT event_id )
{
	portInit();
	enterCritical();
	step = (step << 2) + DT_EVTTRG_FAST_BIT;
	bit_num = bit_num + 2;
	_TP_BusOutFastDrv( step, bit_num );
	portSetCS(0);
	_TP_BusOutByteDrv(event_id);
	_TP_BusOutByteDrv((event_id >> 8) & 0x0f);
	portSetCS(1);
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_EventTrigger32Fast                                              */
/*  Desc:   Called by Event Trigger 32bit Fast Test Point                       */
/*==============================================================================*/
void _TP_EventTrigger32Fast( DT_UINT step, DT_UINT bit_num, DT_UINT event_id )
{
	portInit();
	enterCritical();
	step = (step << 2) + DT_EVTTRG_FAST_BIT;
	bit_num = bit_num + 2;
	_TP_BusOutFastDrv( step, bit_num );
	portSetCS(0);
	_TP_BusOutByteDrv(event_id);
	_TP_BusOutByteDrv(event_id >> 8);
	_TP_BusOutByteDrv(event_id >> 16);
	_TP_BusOutByteDrv(event_id >> 24);
	portSetCS(1);
	exitCritical();
}

/*=============================================================================*/


